/**
 * 
 */
package com.oauth.wrap.api;

import java.util.HashMap;
import java.util.Map;

import com.oauth.wrap.extractor.AccessTokenExtractor;
import com.oauth.wrap.extractor.AccessTokenExtractorImpl;
import com.oauth.wrap.model.Config;
import com.oauth.wrap.model.Constant;
import com.oauth.wrap.model.HttpVerb;
import com.oauth.wrap.model.RefreshToken;
import com.oauth.wrap.model.Verifier;
import com.oauth.wrap.util.URLUtils;

/**
 * Contains all the configuration needed to instantiate a valid {@link Service}
 * 
 * @author Oleksii Karpynskyi <okarpynskyi@gmail.com>
 * 
 */
public abstract class Api {

    /**
     * 
     */
    protected Config config;

    /**
     * Instantiate a OAuth WRAP Service
     * 
     * @param config
     * @return {@link Service}
     */
    public Service createService(Config config) {
        this.config = config;
        return new Service(this, config);
    }

    /**
     * @return {@link Config}
     */
    public Config getConfig() {
        return config;
    }

    /**
     * @return {@link AccessTokenExtractorImpl}
     */
    public AccessTokenExtractor getAccessTokenExtractor() {
        return new AccessTokenExtractorImpl();
    }

    /**
     * Returns the URL where you should redirect your users to authenticate your application.
     * 
     * @return the URL where you should redirect your users
     */
    public abstract String getAuthorizationUrl();

    /**
     * Returns an authentication URL with query string parameters
     * 
     */
    public String getAuthorizationUrlQueryString() {
        return URLUtils.appendParametersToQueryString(getAuthorizationUrl(), getAuthorizationParameters());
    }

    /**
     * 
     */
    public Map<String, String> getAuthorizationParameters() {

        Map<String, String> parameters = new HashMap<String, String>();

        parameters.put(Constant.CLIENT_ID, config.getClientId());
        parameters.put(Constant.CALLBACK, config.getCallback());
        if (config.getScope() != null) {
            parameters.put(Constant.SCOPE, config.getScope());
        }
        if (config.getClientState() != null) {
            parameters.put(Constant.CLIENT_STATE, config.getClientState());
        }

        Map<String, String> extraParameters = config.getExtraParameters();
        if (extraParameters != null) {
            for (Map.Entry<String, String> param : extraParameters.entrySet()) {
                parameters.put(param.getKey(), param.getValue());
            }
        }

        return parameters;
    }

    /**
     * 
     * @return {@link HttpVerb}
     */
    public HttpVerb getAccessTokenHttpVerb() {
        return HttpVerb.POST;
    }

    /**
     * 
     */
    public abstract String getAccessTokenUrl(Verifier verifier);

    /**
     * 
     * @param verifier
     * @return Map
     */
    public Map<String, String> getAccessTokenParameters(Verifier verifier) {

        Map<String, String> parameters = new HashMap<String, String>();

        parameters.put(Constant.CLIENT_ID, config.getClientId());
        parameters.put(Constant.CLIENT_SECRET, config.getClientSecret());
        parameters.put(Constant.CALLBACK, config.getCallback());
        parameters.put(Constant.VERIFICATION_CODE, verifier.getCode());

        Map<String, String> extraParameters = config.getExtraParameters();
        if (extraParameters != null) {
            for (Map.Entry<String, String> param : extraParameters.entrySet()) {
                parameters.put(param.getKey(), param.getValue());
            }
        }

        return parameters;
    }

    /**
     * 
     */
    public Map<String, String> getAccessTokenHeaders(Verifier verifier) {
        Map<String, String> headers = new HashMap<String, String>();
        // Map<String, String> parameters = getAccessTokenParams(verifier);
        headers.put("Content-Type", "application/x-www-form-urlencoded");
        // headers.put("Content-length", ""); // calculate parameters length
        return headers;
    }

    /**
     * 
     * @return {@link HttpVerb}
     */
    public HttpVerb getRefreshTokenHttpVerb() {
        return getAccessTokenHttpVerb();
    }

    /**
     * 
     */
    public abstract String getRefreshTokenUrl();

    /**
     * 
     * @param refreshToken
     * @return Map
     */
    public Map<String, String> getRefreshTokenParameters(RefreshToken refreshToken) {

        Map<String, String> parameters = new HashMap<String, String>();

        parameters.put(Constant.REFRESH_TOKEN, refreshToken.getRefreshToken());

        Map<String, String> extraParameters = config.getExtraParameters();
        if (extraParameters != null) {
            for (Map.Entry<String, String> param : extraParameters.entrySet()) {
                parameters.put(param.getKey(), param.getValue());
            }
        }

        return parameters;
    }

    /**
     * 
     */
    public Map<String, String> getRefreshTokenHeaders(RefreshToken refreshToken) {
        Map<String, String> headers = new HashMap<String, String>();
        // Map<String, String> parameters = getRefreshTokenParams(refreshToken);
        headers.put("Content-Type", "application/x-www-form-urlencoded");
        // headers.put("Content-length", ""); // calculate parameters length
        return headers;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "Api [config=" + config + "]";
    }

}
