/**
 * 
 */
package com.oauth.wrap.api;

import java.util.HashMap;
import java.util.Map;

import com.oauth.wrap.model.Config;
import com.oauth.wrap.util.Preconditions;

/**
 * @author Oleksii Karpynskyi <okarpynskyi@gmail.com>
 * 
 */
public class ServiceBuilder {

    /**
     * 
     */
    private Api api;
    private String clientId;
    private String clientSecret;
    private String callback;
    private String clientState;
    private String scope;
    private Map<String, String> extraParameters;

    /**
     * 
     */
    public ServiceBuilder() {
        extraParameters = new HashMap<String, String>();
    }

    /**
     * Configures the {@link Api}
     * 
     * @param api
     *            instance of {@link Api}s
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder api(Api api) {
        Preconditions.checkNotNull(api, "OAuth WRAP Api cannot be null");
        this.api = api;
        return this;
    }

    /**
     * Adds an OAuth WRAP callback url
     * 
     * @param callback
     *            callback url.
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder callback(String callback) {
        Preconditions.checkValidOAuthWRAPCallback(callback, "Callback must be a valid URL");
        this.callback = callback;
        return this;
    }

    /**
     * Configures the api key
     * 
     * @param clientId
     *            The api key for your application
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder clientId(String clientId) {
        Preconditions.checkEmptyString(clientId, "Invalid OAuth WRAP client id");
        this.clientId = clientId;
        return this;
    }

    /**
     * Configures the client secret
     * 
     * @param clientSecret
     *            The api secret for your application
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder clientSecret(String clientSecret) {
        Preconditions.checkEmptyString(clientSecret, "Invalid OAuth WRAP client secret");
        this.clientSecret = clientSecret;
        return this;
    }

    /**
     * Configures the api client state
     * 
     * @param clientState
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder clientState(String clientState) {
        Preconditions.checkEmptyString(clientState, "Invalid OAuth WRAP client state");
        this.clientState = clientState;
        return this;
    }

    /**
     * Configures the OAuth WRAP scope.
     * 
     * @param scope
     *            The OAuth WRAP scope
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder scope(String scope) {
        Preconditions.checkEmptyString(scope, "Invalid OAuth WRAP scope");
        this.scope = scope;
        return this;
    }

    /**
     * Configures The OAuth WRAP additional parameters
     * 
     * @param name
     * @param value
     * @return the {@link ServiceBuilder} instance for method chaining
     */
    public ServiceBuilder addExtraParameter(String name, String value) {
        Preconditions.checkEmptyString(name, "Invalid OAuth WRAP param name");
        Preconditions.checkEmptyString(value, "Invalid OAuth WRAP param value");
        this.extraParameters.put(name, value);
        return this;
    }

    /**
     * @return the api
     */
    public Api getApi() {
        return api;
    }

    /**
     * @return the clientId
     */
    public String getClientId() {
        return clientId;
    }

    /**
     * @return the clientSecret
     */
    public String getClientSecret() {
        return clientSecret;
    }

    /**
     * @return the callback
     */
    public String getCallback() {
        return callback;
    }

    /**
     * @return the clientState
     */
    public String getClientState() {
        return clientState;
    }

    /**
     * @return the scope
     */
    public String getScope() {
        return scope;
    }

    /**
     * @return the extraParameters
     */
    public Map<String, String> getAddParams() {
        return extraParameters;
    }

    /**
     * Returns the fully configured {@link Service}
     * 
     * @return fully configured {@link Service}
     */
    public Service build() {
        Preconditions.checkNotNull(api, "You must specify a valid api through the provider() method");
        Preconditions.checkEmptyString(clientId, "You must provide an client id");
        Preconditions.checkEmptyString(clientSecret, "You must provide an client secret");
        Config config = new Config(clientId, clientSecret, callback, clientState, scope);
        if (extraParameters != null) {
            config.setExtraParameters(extraParameters);
        }
        return api.createService(config);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "ServiceBuilder [api=" + api + ", clientId=" + clientId + ", clientSecret=" + clientSecret + ", callback=" + callback + ", clientState=" + clientState + ", scope=" + scope + ", extraParameters=" + extraParameters + "]";
    }

}
