package com.oauth.wrap.model;

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import com.oauth.wrap.exception.OAuthWRAPException;
import com.oauth.wrap.util.URLUtils;

/**
 * Represents an HTTP Request object
 * 
 * @author Oleksii Karpynskyi <okarpynskyi@gmail.com>
 */
public class Request {
    private static final String CONTENT_LENGTH = "Content-Length";

    private String url;
    private HttpVerb httpVerb;
    private Map<String, String> querystringParams;
    private Map<String, String> bodyParams;
    private Map<String, String> headers;
    private String payload = null;
    private HttpURLConnection connection;
    private String charset;

    /**
     * Default constructor
     */
    public Request() {
        this(null, null);
    }

    /**
     * Creates a new Http Request
     * 
     * @param httpVerb
     *            Http httpVerb (GET, POST, etc)
     * @param url
     *            url with optional querystring parameters.
     */
    public Request(HttpVerb httpVerb, String url) {
        this.httpVerb = httpVerb;
        this.url = url;
        this.querystringParams = new HashMap<String, String>();
        this.bodyParams = new HashMap<String, String>();
        this.headers = new HashMap<String, String>();
    }

    /**
     * Execute the request and return a {@link Response}
     * 
     * @return Http Response
     * @throws RuntimeException
     *             if the connection cannot be created.
     */
    public Response send() {
        try {
            createConnection();
            return doSend();
        } catch (IOException ioe) {
            throw new OAuthWRAPException("Problems while creating connection", ioe);
        }
    }

    private void createConnection() throws IOException {
        String effectiveUrl = url;
        if (httpVerb.equals(HttpVerb.PUT) || httpVerb.equals(HttpVerb.POST)) {
            effectiveUrl = URLUtils.appendParametersToQueryString(url, querystringParams);
        }
        if (connection == null) {
            System.setProperty("http.keepAlive", "false");
            connection = (HttpURLConnection) new URL(effectiveUrl).openConnection();
        }
    }

    @SuppressWarnings("unused")
    Response doSend() throws IOException {
        connection.setRequestMethod(this.httpVerb.name());
        addHeaders(connection);
        if (httpVerb.equals(HttpVerb.PUT) || httpVerb.equals(HttpVerb.POST)) {
            addBody(connection, getBodyContents());
            // send POST params manualy
            if (false && !querystringParams.isEmpty()) {
                // Create I/O streams and send POST parameters
                DataOutputStream outStream = new DataOutputStream(connection.getOutputStream());
                outStream.writeBytes(URLUtils.formURLEncodeMap(querystringParams));
                outStream.flush();
                outStream.close();
            }
        }

        return new Response(connection);
    }

    void addHeaders(HttpURLConnection conn) {
        for (String key : headers.keySet())
            conn.setRequestProperty(key, headers.get(key));
    }

    void addBody(HttpURLConnection conn, String content) throws IOException {
        if (this.charset == null)
            this.charset = Charset.defaultCharset().name();
        conn.setRequestProperty(CONTENT_LENGTH, String.valueOf(content.getBytes(charset).length));
        conn.setDoOutput(true);
        conn.getOutputStream().write(content.getBytes(charset));
    }

    /**
     * Add an HTTP Header to the Request
     * 
     * @param key
     *            the header name
     * @param value
     *            the header value
     */
    public void addHeader(String key, String value) {
        this.headers.put(key, value);
    }

    /**
     * Add HTTP headers
     * 
     * @param headers
     */
    public void addHeaders(Map<String, String> headers) {
        this.headers.putAll(headers);
    }

    /**
     * Add a body Parameter (for POST/ PUT Requests)
     * 
     * @param key
     *            the parameter name
     * @param value
     *            the parameter value
     */
    public void addBodyParameter(String key, String value) {
        this.bodyParams.put(key, value);
    }

    /**
     * Add a body Parameters (for POST/ PUT Requests)
     * 
     * @param parameters
     */
    public void addBodyParameters(Map<String, String> parameters) {
        this.bodyParams.putAll(parameters);
    }

    /**
     * Add a QueryString parameter
     * 
     * @param key
     *            the parameter name
     * @param value
     *            the parameter value
     */
    public void addQuerystringParameter(String key, String value) {
        this.querystringParams.put(key, value);
    }

    /**
     * Add QueryString parameters
     * 
     * @param parameters
     */
    public void addQuerystringParameters(Map<String, String> parameters) {
        this.querystringParams.putAll(parameters);
    }

    /**
     * Add body payload.
     * 
     * This method is used when the HTTP body is not a form-url-encoded string, but another thing. Like for example XML.
     * 
     * Note: The contents are not part of the OAuth signature
     * 
     * @param payload
     *            the body of the request
     */
    public void addPayload(String payload) {
        this.payload = payload;
    }

    /**
     * Get a {@link Map} of the query string parameters.
     * 
     * @return a map containing the query string parameters
     * @throws OAuthWRAPException
     *             if the URL is not valid
     */
    public Map<String, String> getQueryStringParams() {
        try {
            Map<String, String> params = new HashMap<String, String>();
            String queryString = new URL(url).getQuery();
            params.putAll(URLUtils.queryStringToMap(queryString));
            params.putAll(this.querystringParams);
            return params;
        } catch (MalformedURLException mue) {
            throw new OAuthWRAPException("Malformed URL", mue);
        }
    }

    /**
     * Obtains a {@link Map} of the body parameters.
     * 
     * @return a map containing the body parameters.
     */
    public Map<String, String> getBodyParams() {
        return bodyParams;
    }

    /**
     * Obtains the URL of the HTTP Request.
     * 
     * @return the original URL of the HTTP Request
     */
    public String getUrl() {
        return url;
    }

    /**
     * Returns the URL without the port and the query string part.
     * 
     * @return the OAuth-sanitized URL
     */
    public String getSanitizedUrl() {
        return url.replaceAll("\\?.*", "").replace("\\:\\d{4}", "");
    }

    /**
     * Returns the body of the request
     * 
     * @return form encoded string
     */
    public String getBodyContents() {
        return (payload != null) ? payload : URLUtils.formURLEncodeMap(bodyParams);
    }

    /**
     * Returns the HTTP httpVerb
     * 
     * @return the httpVerb
     */
    public HttpVerb getHttpVerb() {
        return httpVerb;
    }

    /**
     * Returns the connection headers as a {@link Map}
     * 
     * @return map of headers
     */
    public Map<String, String> getHeaders() {
        return headers;
    }

    /**
     * Sets the connect timeout for the underlying {@link HttpURLConnection}
     * 
     * @param duration
     *            duration of the timeout
     * 
     * @param unit
     *            unit of time (milliseconds, seconds, etc)
     */
    public void setConnectTimeout(int duration, TimeUnit unit) {
        this.connection.setConnectTimeout((int) unit.toMillis(duration));
    }

    /**
     * Sets the read timeout for the underlying {@link HttpURLConnection}
     * 
     * @param duration
     *            duration of the timeout
     * 
     * @param unit
     *            unit of time (milliseconds, seconds, etc)
     */
    public void setReadTimeout(int duration, TimeUnit unit) {
        this.connection.setReadTimeout((int) unit.toMillis(duration));
    }

    /**
     * Set the charset of the body of the request
     * 
     * @param charsetName
     *            name of the charset of the request
     */
    public void setCharset(String charsetName) {
        this.charset = charsetName;
    }

    /*
     * We need this in order to stub the connection object for test cases
     */
    void setConnection(HttpURLConnection connection) {
        this.connection = connection;
    }

    /**
     * @param url
     *            the url to set
     */
    public void setUrl(String url) {
        this.url = url;
    }

    /**
     * @param httpVerb
     *            the httpVerb to set
     */
    public void setHttpVerb(HttpVerb httpVerb) {
        this.httpVerb = httpVerb;
    }

    /**
     * @return the querystringParams
     */
    public Map<String, String> getQuerystringParams() {
        return querystringParams;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        return "Request [url=" + url + ", httpVerb=" + httpVerb + ", querystringParams=" + querystringParams + ", bodyParams=" + bodyParams + ", headers=" + headers + ", payload=" + payload + ", connection=" + connection + ", charset=" + charset + "]";
    }

}
